#include <hiredis/hiredis.h>
#include <string.h>
#include <stdlib.h>
#include "clib.h"
#include <assert.h>
#include <math.h>
#include <stdarg.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <string.h>
#include<sys/socket.h>
#include<errno.h> //For errno - the error number
#include<netdb.h>	//hostent
#include<arpa/inet.h>
#include <random>
#include <pqxx/pqxx>
#include <time.h>
#include <map>

#define DBG_HERE 0

static char curr_time[1 << 10];
static time_t start_time_t, curr_time_t;
static double start_time_d, curr_time_d, rem_time_d, advance_elapsed_glob = 0;
int init_rand_seed;

void init_time()
{
  start_time_t = time((time_t *)0);
  curr_time_t = start_time_t;
  start_time_d = (double)start_time_t;
  curr_time_d = (double)start_time_t;
  rem_time_d = 0;
  advance_elapsed_glob = 0;
}

static void advance_time(double t)
{
  curr_time_d = start_time_d + t;
  rem_time_d = curr_time_d - floor(curr_time_d);
  curr_time_t = (time_t)floor(curr_time_d);
}

char *get_curr_time_psql()
{
  char *tmp = ctime(&curr_time_t);
  tmp[strlen(tmp) - 1] = '\0'; /* override the \n */
  sprintf(curr_time, "TIMESTAMP WITHOUT TIME ZONE '%s' + INTERVAL '%lf seconds'", tmp, rem_time_d);
  return curr_time;
}

redisContext *conn_to_redis(char **argv)
{
  const char *hostname = argv[1];
  const int port = atoi(argv[2]);
  struct timeval timeout = { 1, 500000 }; // 1.5 seconds
  redisContext *conn = redisConnectWithTimeout(hostname, port, timeout);
  if (conn == NULL || conn->err) {
    if (conn) {
      printf("Connection error: %s\n", conn->errstr);
      redisFree(conn);
    } else {
      printf("Connection error: can't allocate redis context\n");
    }
    exit(1);
  }
  return conn;
}

void from_double_to_timespec(struct timespec *req, double t)
{
  req->tv_sec = (time_t)floor(t);
  req->tv_nsec = ((long)(((t - floor(t)))*1000000000L))%1000000000L;
}

void print_all(int dbg, const char *str, ...)
{
  if (dbg) {
    struct timespec tv;
    clock_gettime(CLOCK_MONOTONIC, &tv);
    printf("%ld.%ld: ", tv.tv_sec, tv.tv_nsec);
    va_list arg;
    va_start(arg, str);
    vprintf(str, arg);
    va_end(arg);
    fflush(stdout);
  }
}

char *hostname_to_ip(const char * hostname)
{
  struct hostent *he;
  struct in_addr **addr_list;
  int i;
		
  if ( (he = gethostbyname( hostname ) ) == NULL) 
    {
      // get the host info
      herror("gethostbyname");
      return (char *)0;
    }

  addr_list = (struct in_addr **) he->h_addr_list;
	
  for(i = 0; addr_list[i] != NULL; i++) 
    {
      //Return the first one;
      return strdup(inet_ntoa(*addr_list[i]) );
    }
	
  return (char *)0;
}

static int compare(const void* a, const void* b)
{
  const double *x = (double *)a;
  const double *y = (double *)b;
  if (*x > *y)
    return 1;
  else if (*x < *y)
    return -1;
  return 0;
}

void init_rand_monitored_sleep(int s)
{
  init_rand_seed = s;
}

#define VALS_MIN_SIZE (1 << 10)
void monitored_sleep(redisContext *conn, const char *function, double t_arg, unsigned type, ...)
{
  static pqxx::connection *C = (pqxx::connection *)0;
  static pqxx::nontransaction *N = (pqxx::nontransaction *)0;
  static std::mt19937 *gen; // mersenne_twister_engine seeded with rd()
  static char sql[1024];
  static std::map<std::string, double *> times = {};
  static std::map<std::string, unsigned> times_len = {};
  struct timespec to_be_slept_tv;
  if (!C) {
    gen = new std::mt19937(init_rand_seed);
    char *tmp = hostname_to_ip(ENV_DBHOSTNAME);
    sprintf(sql, "dbname = %s user = %s password = %s hostaddr = %s port = %d", ENV_DBNAME, ENV_DBUSER, ENV_DBPASSWORD, tmp, ENV_DBPORT);
    free(tmp);
    C = new pqxx::connection(sql);
    N = new pqxx::nontransaction(*C);
  }
  if (type == 0) {
    from_double_to_timespec(&to_be_slept_tv, t_arg + advance_elapsed_glob);
    nanosleep(&to_be_slept_tv, NULL);
    advance_time(t_arg + advance_elapsed_glob);
    advance_elapsed_glob = 0;
  }
  else {
    std::string all_args = "";
    va_list strings;
    va_start(strings, type);
    unsigned i;
    for (i = 0; i < type; i++) {
      char *s = va_arg(strings, char *);
      all_args.append(s);
      all_args.append("|");
    }
    va_end(strings);
    if (times.count(all_args) == 0) {
      double *vals = (double *)0;
      unsigned vals_len = VALS_MIN_SIZE;
      unsigned vals_curr_len = 0;
      if (!vals) 
	vals = (double *)calloc(VALS_MIN_SIZE, sizeof(double));
      va_start(strings, type);
      unsigned j, k, l;
      for (i = 0; i < type; i++) {
	char *s = va_arg(strings, char *);
	sprintf(sql, "SELECT single_time FROM function_time INNER JOIN function_info ON function_info.id = function_id WHERE fname = '%s' AND statm_type = '%s'", function, s);
	pqxx::result R2(N->exec(sql));
	if (vals_curr_len == 0) {
	  if (vals_len < R2.size()) {
	    vals_len = (R2.size()/VALS_MIN_SIZE + 1)*VALS_MIN_SIZE;
	    vals = (double *)realloc(vals, vals_len*sizeof(double));
	  }
	  j = 0;
	  for (pqxx::result::const_iterator c = R2.begin(); c != R2.end(); ++c) 
	    vals[j++] = c[0].as<double>();
	  vals_curr_len = j;
	}
	else {
	  if (vals_len < R2.size()*vals_curr_len + vals_curr_len) {
	    vals_len = ((R2.size()*vals_curr_len + vals_curr_len)/VALS_MIN_SIZE + 1)*VALS_MIN_SIZE;
	    vals = (double *)realloc(vals, vals_len*sizeof(double));
	  }
	  for (j = 0; j < vals_curr_len; j++)
	    vals[j + R2.size()*vals_curr_len] = vals[j];
	  for (j = 0; j < vals_curr_len; j++) {
	    for (pqxx::result::const_iterator c = R2.begin(); c != R2.end(); ++c) 
	      vals[j] = vals[j + R2.size()*vals_curr_len] + c[0].as<double>();
	  }
	}
	qsort(vals, vals_curr_len, sizeof(double), compare);
	for (j = 0; j < vals_curr_len; j++) {
	  unsigned k;
	  for (k = j + 1; k < vals_curr_len && vals[k] == vals[j]; k++);
	  if (k != j + 1) {
	    /*
	      vals_curr_len = 8
	      1 4 6 6 6 6 6 8
	      j = 2
	      k = 7
	      1 4 6 8
	      vals_curr_len = 4
	    */
	    for (unsigned l = k; l < vals_curr_len; l++)
	      vals[l - k + j + 1] = vals[l];
	    vals_curr_len -= k - j - 1;
	  }
	}
      }
      va_end(strings);
      times[all_args] = vals;
      times_len[all_args] = vals_curr_len;
    }
    std::uniform_int_distribution<> distrib(0, times_len[all_args] - 1);
    from_double_to_timespec(&to_be_slept_tv, advance_elapsed_glob + times[all_args][distrib(*gen)]);
    nanosleep(&to_be_slept_tv, NULL);
    advance_time(advance_elapsed_glob + times[all_args][distrib(*gen)]);
    advance_elapsed_glob = 0;
  }
}

void only_advance_time_monitored_sleep(redisContext *conn, const char *function, unsigned type, ...)
{
  static pqxx::connection *C = (pqxx::connection *)0;
  static pqxx::nontransaction *N = (pqxx::nontransaction *)0;
  static char sql[1024];
  static std::mt19937 *gen; // mersenne_twister_engine seeded with rd()
  static std::map<std::string, double *> times = {};
  static std::map<std::string, unsigned> times_len = {};
  if (!C) {
    char *tmp = hostname_to_ip(ENV_DBHOSTNAME);
    sprintf(sql, "dbname = %s user = %s password = %s hostaddr = %s port = %d", ENV_DBNAME, ENV_DBUSER, ENV_DBPASSWORD, tmp, ENV_DBPORT);
    free(tmp);
    C = new pqxx::connection(sql);
    gen = new std::mt19937(init_rand_seed);
    N = new pqxx::nontransaction(*C);
  }
  std::string all_args = "";
  va_list strings;
  va_start(strings, type);
  unsigned i;
  for (i = 0; i < type; i++) {
    char *s = va_arg(strings, char *);
    all_args.append(s);
    all_args.append("|");
  }
  va_end(strings);
  if (times.count(all_args) == 0) {
    double *vals = (double *)0;
    unsigned vals_len = VALS_MIN_SIZE;
    unsigned vals_curr_len = 0;
    if (!vals) 
      vals = (double *)calloc(VALS_MIN_SIZE, sizeof(double));
    va_start(strings, type);
    unsigned j, k, l;
    for (i = 0; i < type; i++) {
      char *s = va_arg(strings, char *);
      sprintf(sql, "SELECT single_time FROM function_time INNER JOIN function_info ON function_info.id = function_id WHERE fname = '%s' AND statm_type = '%s'", function, s);
      pqxx::result R2(N->exec(sql));
      if (vals_curr_len == 0) {
	if (vals_len < R2.size()) {
	  vals_len = (R2.size()/VALS_MIN_SIZE + 1)*VALS_MIN_SIZE;
	  vals = (double *)realloc(vals, vals_len*sizeof(double));
	}
	j = 0;
	for (pqxx::result::const_iterator c = R2.begin(); c != R2.end(); ++c) 
	  vals[j++] = c[0].as<double>();
	vals_curr_len = j;
      }
      else {
	if (vals_len < R2.size()*vals_curr_len + vals_curr_len) {
	  vals_len = ((R2.size()*vals_curr_len + vals_curr_len)/VALS_MIN_SIZE + 1)*VALS_MIN_SIZE;
	  vals = (double *)realloc(vals, vals_len*sizeof(double));
	}
	for (j = 0; j < vals_curr_len; j++)
	  vals[j + R2.size()*vals_curr_len] = vals[j];
	for (j = 0; j < vals_curr_len; j++) {
	  for (pqxx::result::const_iterator c = R2.begin(); c != R2.end(); ++c) 
	    vals[j] = vals[j + R2.size()*vals_curr_len] + c[0].as<double>();
	}
      }
      qsort(vals, vals_curr_len, sizeof(double), compare);
      for (j = 0; j < vals_curr_len; j++) {
	unsigned k;
	for (k = j + 1; k < vals_curr_len && vals[k] == vals[j]; k++);
	if (k != j + 1) {
	  /*
	    vals_curr_len = 8
	    1 4 6 6 6 6 6 8
	    j = 2
	    k = 7
	    1 4 6 8
	    vals_curr_len = 4
	  */
	  for (unsigned l = k; l < vals_curr_len; l++)
	    vals[l - k + j + 1] = vals[l];
	  vals_curr_len -= k - j - 1;
	}
      }
    }
    va_end(strings);
    times[all_args] = vals;
    times_len[all_args] = vals_curr_len;
  }
  // std::random_device rd;  // a seed source for the random number engine
  // std::mt19937 gen(rd()); // mersenne_twister_engine seeded with rd()
  std::uniform_int_distribution<> distrib(0, times_len[all_args] - 1);
  advance_elapsed_glob += times[all_args][distrib(*gen)];
}

char *blocking_read(redisContext *conn, const char *queue, const char *function)
{
  char *ret = (char *)0;
  redisReply *reply = (redisReply *)redisCommand(conn, "BRPOP %s 0", queue);
  ret = strdup(reply->element[1]->str);
  freeReplyObject(reply);
  return ret;
}

int check_assumptions(double tau_s, double **T, unsigned T_num, unsigned *size_T)
{
  int ret = (tau_s > 0);
  for (int i = 0; ret && (i < T_num); i++)
    for (int j = 0; ret && (j < size_T[i]); j++)
      ret = ret && (T[i][j] > 0);
  return ret;
}
