import common_lib
import sys
import argparse
import datetime
import os
import json
import redis
import more_itertools
import random
import math

def main():
    args = parse_args()
    common_lib.print_cmd_line(sys.argv)
    common_lib.print_all_options(args)
    common_lib.init_rand_mon_sleep(args.rand_seed_mon_sleep)
    (conn, db) = common_lib.db_connect("dbname=%s host=%s user=%s password=%s port=%s" %(args.dbname, args.dbhost, args.dbuser, args.dbpassword, args.dbport))
    (conn_env, db_env) = common_lib.db_connect("dbname=%s host=%s user=%s password=%s port=%s" %(args.dbenvname, args.dbenvhost, args.dbenvuser, args.dbenvpassword, args.dbenvport))
    rc = redis.Redis(host=args.redishost, port=args.redisport, decode_responses=True)
    index_cust_orig = args.index_cust
    I = get_items(db)
    while True:
        # elapsed = 0
        J = pick_subset(I, args.max_items)
        for j in J:
            (N, D) = get_n_delay(db_env, j, args.index_cust)
            rc.lpush(args.queue_co_base + str(args.index_cust), str(j))
            (m, tmp) = common_lib.blocking_read(rc, db_env, args.queue_ci_base + str(args.index_cust), "customer_env" + str(args.index_cust))
            # elapsed += tmp
            m = int(m)
            n = random.choice(N)
            r = random.uniform(0, 1)
            if (m >= n) or (r > args.f):
                rc.lpush(args.queue_co_base + str(args.index_cust), str(j) + " " + str(n) + " " + str(random.choice(D)))
                (ign, tmp) = common_lib.blocking_read(rc, db_env, args.queue_ci_base + str(args.index_cust), "customer_env" + str(args.index_cust))
                # elapsed += tmp
        common_lib.monitored_sleep_check(rc, db_env, "customer_env" + str(args.index_cust), cause=["main_sleep"])

def pick_subset(I, maxl):
    hm = random.randint(1, min(len(I), maxl))
    ret = []
    for i in range(hm):
        while True:
            pick = random.choice(I)
            if pick not in ret:
                ret += [pick]
                break
    return ret
                
def get_items(db):
    sql = "SELECT id FROM item"
    res = common_lib.exec_and_retrieve(db, sql)
    return [r["id"] for r in res]

def get_n_delay(db_env, item_id, index_cust):
    sql = "SELECT order_num, order_date_due FROM item_customer_other_info WHERE item_id = %d AND customer_id = %d" %(item_id, index_cust)
    res = common_lib.exec_and_retrieve(db_env, sql)
    ret = [[], []]
    for r in res:
        if r["order_num"] != None:
            ret[0] += [r["order_num"]]
        if r["order_date_due"] != None:
            ret[1] += [r["order_date_due"]]
    return ret

def parse_args():
    QUEUE_CO_BASE_DEF = "work:queue:CO"
    QUEUE_CI_BASE_DEF = "work:queue:CI"
    F_DEF = 0.8
    INDEX_CUST_DEF = 0
    MAX_ITEMS_DEF = 5
    RAND_SEED_MON_SLEEP_DEF = 10

    parser = argparse.ArgumentParser(description='Command arguments')
    parser.add_argument("-r", "--rand_seed_mon_sleep", dest = "rand_seed_mon_sleep", help = "rand_seed_mon_sleep value (default is " + str(RAND_SEED_MON_SLEEP_DEF) + ")", default = RAND_SEED_MON_SLEEP_DEF, type = int)
    parser.add_argument("-q", "--queue_co_base", dest = "queue_co_base", help = "queue_co_base value (default is " + str(QUEUE_CO_BASE_DEF) + ")", default = QUEUE_CO_BASE_DEF, type = str)
    parser.add_argument("-Q", "--queue_ci_base", dest = "queue_ci_base", help = "queue_ci_base value (default is " + str(QUEUE_CI_BASE_DEF) + ")", default = QUEUE_CI_BASE_DEF, type = str)
    parser.add_argument("-i", "--index_cust", dest = "index_cust", help = "index_cust value (default is " + str(INDEX_CUST_DEF) + ")", default = INDEX_CUST_DEF, type = int)
    parser.add_argument("-f", "--f", dest = "f", help = "f value (default is " + str(F_DEF) + ")", default = F_DEF, type = float)
    parser.add_argument("-M", "--max_items", dest = "max_items", help = "max_items value (default is " + str(MAX_ITEMS_DEF) + ")", default = MAX_ITEMS_DEF, type = int)
    parser.add_argument("redishost", help = "redishost", type = str)
    parser.add_argument("redisport", help = "redisport", type = str)
    parser.add_argument("dbname", help = "dbname", type = str)
    parser.add_argument("dbuser", help = "dbuser", type = str)
    parser.add_argument("dbpassword", help = "dbpassword", type = str)
    parser.add_argument("dbhost", help = "dbhost", type = str)
    parser.add_argument("dbport", help = "dbport", type = str)
    parser.add_argument("dbenvname", help = "dbenvname", type = str)
    parser.add_argument("dbenvuser", help = "dbenvuser", type = str)
    parser.add_argument("dbenvpassword", help = "dbenvpassword", type = str)
    parser.add_argument("dbenvhost", help = "dbenvhost", type = str)
    parser.add_argument("dbenvport", help = "dbenvport", type = str)

    args = parser.parse_args()

    if args.queue_co_base == "":
        print("Error: queue_co_base (-q) must not be the empty string")
        sys.exit(2)

    if args.queue_ci_base == "":
        print("Error: queue_ci_base (-Q) must not be the empty string")
        sys.exit(2)

    if args.max_items <= 0:
        print("Error: max_items (-m) must be positive")
        sys.exit(2)

    if args.f < 0 or args.f > 1:
        print("Error: f (-f) must not be in [0, 1]")
        sys.exit(2)

    if args.index_cust < 0:
        print("Error: index_cust (-i) must not be negative")
        sys.exit(2)

    return args
    
main()
