import common_lib
import sys
import argparse
import datetime
import os
import json
import redis
import more_itertools
import random

def main():
    args = parse_args()
    common_lib.print_cmd_line(sys.argv)
    common_lib.print_all_options(args)
    common_lib.init_rand_mon_sleep(args.rand_seed_mon_sleep)
    (conn, db) = common_lib.db_connect("dbname=%s host=%s user=%s password=%s port=%s" %(args.dbname, args.dbhost, args.dbuser, args.dbpassword, args.dbport))
    (conn_env, db_env) = common_lib.db_connect("dbname=%s host=%s user=%s password=%s port=%s" %(args.dbenvname, args.dbenvhost, args.dbenvuser, args.dbenvpassword, args.dbenvport))
    rc = redis.Redis(host=args.redishost, port=args.redisport, decode_responses=True)
    (I, p) = get_items(db, db_env, args.index_prod)
    pp = [0] * len(p)
    # times = [0.1] * 4
    while True:
        # elapsed = 0
        tmp = rc.llen(args.queue_pii_base + str(args.index_prod))
        if tmp > 0:
            X = rc.rpop(args.queue_pii_base + str(args.index_prod))
            Xar = X.split()
            i = int(Xar[0])
            (T, l) = get_time_l(db_env, i, args.index_prod)
            if len(Xar) == 2:
                book = min(p[corr(I, i)], int(Xar[1]))
                rc.lpush(args.queue_poi_base + str(args.index_prod), str(book))
                p[corr(I, i)] -= book
            else:
                rc.lpush(args.queue_poi_base + str(args.index_prod), str(p[corr(I, i)]))
            # elapsed += times[0]
        tmp = rc.llen(args.queue_pip_base + str(args.index_prod))
        if tmp > 0:
            X = rc.rpop(args.queue_pip_base + str(args.index_prod))
            Xar = X.split()
            n = int(Xar[2])
            o = int(Xar[1])
            if n - p[corr(I, i)] > 0:
                rc.lpush(args.queue_ppi_base + str(args.index_prod), "4 " + str(i) + " " + str(n - p[corr(I, i)]) + " " + str(o) + " " + str(T).replace("[", "").replace("]", "").replace(",", ""))
                p[corr(I, i)] = 0
            else:
                p[corr(I, i)] -= n
                rc.lpush(args.queue_pop_base + str(args.index_prod), str(o) + " " + str(n))
            # elapsed += times[1]
        for i in I:
            (T, l) = get_time_l(db_env, i, args.index_prod)
            if l > p[corr(I, i)] + pp[corr(I, i)]:
                rc.lpush(args.queue_ppi_base + str(args.index_prod), "3 " + str(i) + " " + str(l - p[corr(I, i)]) + " " + str(T).replace("[", "").replace("]", "").replace(",", ""))
                pp[corr(I, i)] = l - p[corr(I, i)]
                # elapsed += times[2]
        tmp = rc.llen(args.queue_ppo_base + str(args.index_prod))
        if tmp > 0:
            X = rc.rpop(args.queue_ppo_base + str(args.index_prod))
            Xar = X.split()
            if Xar[0] == "3":
                rc.lpush(args.queue_pop_base + str(args.index_prod), Xar[2] + " " + Xar[1])
            else:
                p[corr(I, i)] = int(Xar[1])
                pp[corr(I, i)] = 0
            # elapsed += times[3]
        common_lib.monitored_sleep_check(rc, db_env, "producer_env" + str(args.index_prod), cause=["main_sleep"])

def corr(I, i):
    return I.index(i)
                
def get_items(db, db_env, index_prod):
    sql = "SELECT item.id FROM item INNER JOIN item_producer ON item_id = item.id WHERE producer_id = %d ORDER BY item.id" %index_prod 
    res1 = common_lib.exec_and_retrieve(db, sql)
    sql = "SELECT item_id, prod_starting FROM item_producer_other_info WHERE producer_id = %d AND prod_starting IS NOT NULL ORDER BY item_id" %index_prod 
    res2 = common_lib.exec_and_retrieve(db_env, sql)
    assert(len(res1) == len(res2))
    for i in range(len(res1)):
        assert(res1[i]["id"] == res2[i]["item_id"])
    return ([r["id"] for r in res1], [r["prod_starting"] for r in res2])

def get_time_l(db_env, item_id, index_prod):
    sql = "SELECT prod_min, prod_time FROM item_producer_other_info WHERE item_id = %d AND producer_id = %d" %(item_id, index_prod)
    res = common_lib.exec_and_retrieve(db_env, sql)
    ret = []
    for r in res:
        if r["prod_min"] != None:
            l = r["prod_min"]
        ret += [r["prod_time"]]
    return (ret, l)
        
def load_params(filename):
    f = open(filename)
    data = json.load(f)
    f.close()
    return data

def parse_args():
    QUEUE_POI_BASE_DEF = "work:queue:POI"
    QUEUE_PII_BASE_DEF = "work:queue:PII"
    QUEUE_POP_BASE_DEF = "work:queue:POP"
    QUEUE_PIP_BASE_DEF = "work:queue:PIP"
    QUEUE_PPO_BASE_DEF = "work:queue:PPO"
    QUEUE_PPI_BASE_DEF = "work:queue:PPI"
    INDEX_PROD_DEF = 0
    RAND_SEED_MON_SLEEP_DEF = 10

    parser = argparse.ArgumentParser(description='Command arguments')
    parser.add_argument("-r", "--rand_seed_mon_sleep", dest = "rand_seed_mon_sleep", help = "rand_seed_mon_sleep value (default is " + str(RAND_SEED_MON_SLEEP_DEF) + ")", default = RAND_SEED_MON_SLEEP_DEF, type = int)
    parser.add_argument("-q", "--queue_poi_base", dest = "queue_poi_base", help = "queue_poi_base value (default is " + str(QUEUE_POI_BASE_DEF) + ")", default = QUEUE_POI_BASE_DEF, type = str)
    parser.add_argument("-Q", "--queue_pii_base", dest = "queue_pii_base", help = "queue_pii_base value (default is " + str(QUEUE_PII_BASE_DEF) + ")", default = QUEUE_PII_BASE_DEF, type = str)
    parser.add_argument("-s", "--queue_pop_base", dest = "queue_pop_base", help = "queue_pop_base value (default is " + str(QUEUE_POP_BASE_DEF) + ")", default = QUEUE_POP_BASE_DEF, type = str)
    parser.add_argument("-S", "--queue_pip_base", dest = "queue_pip_base", help = "queue_pip_base value (default is " + str(QUEUE_PIP_BASE_DEF) + ")", default = QUEUE_PIP_BASE_DEF, type = str)
    parser.add_argument("-p", "--queue_ppo_base", dest = "queue_ppo_base", help = "queue_ppo_base value (default is " + str(QUEUE_PPO_BASE_DEF) + ")", default = QUEUE_PPO_BASE_DEF, type = str)
    parser.add_argument("-P", "--queue_ppi_base", dest = "queue_ppi_base", help = "queue_ppi_base value (default is " + str(QUEUE_PPI_BASE_DEF) + ")", default = QUEUE_PPI_BASE_DEF, type = str)
    parser.add_argument("-i", "--index_prod", dest = "index_prod", help = "index_prod value (default is " + str(INDEX_PROD_DEF) + ")", default = INDEX_PROD_DEF, type = int)
    parser.add_argument("redishost", help = "redishost", type = str)
    parser.add_argument("redisport", help = "redisport", type = str)
    parser.add_argument("dbname", help = "dbname", type = str)
    parser.add_argument("dbuser", help = "dbuser", type = str)
    parser.add_argument("dbpassword", help = "dbpassword", type = str)
    parser.add_argument("dbhost", help = "dbhost", type = str)
    parser.add_argument("dbport", help = "dbport", type = str)
    parser.add_argument("dbenvname", help = "dbenvname", type = str)
    parser.add_argument("dbenvuser", help = "dbenvuser", type = str)
    parser.add_argument("dbenvpassword", help = "dbenvpassword", type = str)
    parser.add_argument("dbenvhost", help = "dbenvhost", type = str)
    parser.add_argument("dbenvport", help = "dbenvport", type = str)

    args = parser.parse_args()

    if args.queue_pop_base == "":
        print("Error: queue_pop_base (-r) must not be the empty string")
        sys.exit(2)

    if args.queue_pip_base == "":
        print("Error: queue_pip_base (-R) must not be the empty string")
        sys.exit(2)

    if args.queue_poi_base == "":
        print("Error: queue_poi_base (-q) must not be the empty string")
        sys.exit(2)

    if args.queue_pii_base == "":
        print("Error: queue_pii_base (-Q) must not be the empty string")
        sys.exit(2)

    if args.queue_ppo_base == "":
        print("Error: queue_ppo_base (-p) must not be the empty string")
        sys.exit(2)

    if args.queue_ppi_base == "":
        print("Error: queue_ppi_base (-P) must not be the empty string")
        sys.exit(2)

    if args.index_prod < 0:
        print("Error: index_prod (-i) must not be negative")
        sys.exit(2)

    return args
    
main()
