import common_lib
import sys
import argparse
import datetime
import os
import json
import redis
import more_itertools
import random

def main():
    args = parse_args()
    common_lib.print_cmd_line(sys.argv)
    common_lib.print_all_options(args)
    common_lib.init_rand_mon_sleep(args.rand_seed_mon_sleep)
    (conn_env, db_env) = common_lib.db_connect("dbname=%s host=%s user=%s password=%s port=%s" %(args.dbenvname, args.dbenvhost, args.dbenvuser, args.dbenvpassword, args.dbenvport))
    rc = redis.Redis(host=args.redishost, port=args.redisport, decode_responses=True)
    while True:
        tmp = rc.llen(args.queue_ppi_base + str(args.index_prod))
        if tmp > 0:
            X = rc.rpop(args.queue_ppi_base + str(args.index_prod))
            if X == None: #ready for multi-phys; however, producer is not ready for multi-phys
                continue
            Xar = X.split()
            if (Xar[0] == "4"):
                o = Xar[3]
                T_from = 4
            else:
                o = None
                T_from = 3
            i = Xar[1]
            m = int(Xar[2])
            T = [float(t) for t in Xar[T_from : ]]
            M = sum([random.choice(T) for j in range(m)])
            common_lib.monitored_sleep_check(rc, db_env, "producer_phys_env" + str(args.index_prod), fixed_time=M)
            rc.lpush(args.queue_ppo_base + str(args.index_prod), str(int(Xar[0]) - 1) + " " + i + " " + str(m) + ("" if o == None else o))
        common_lib.monitored_sleep_check(rc, db_env, "producer_phys_env" + str(args.index_prod), cause=["main_sleep"])

def parse_args():
    QUEUE_PPO_BASE_DEF = "work:queue:PPO"
    QUEUE_PPI_BASE_DEF = "work:queue:PPI"
    # TILDE_T_DEF = str([0.1*i for i in range(1, 11)]).replace(", ", ":").replace("[", "").replace("]", "")
    INDEX_PROD_DEF = 0
    RAND_SEED_MON_SLEEP_DEF = 10

    parser = argparse.ArgumentParser(description='Command arguments')
    parser.add_argument("-q", "--queue_ppo_base", dest = "queue_ppo_base", help = "queue_ppo_base value (default is " + str(QUEUE_PPO_BASE_DEF) + ")", default = QUEUE_PPO_BASE_DEF, type = str)
    parser.add_argument("-Q", "--queue_ppi_base", dest = "queue_ppi_base", help = "queue_ppi_base value (default is " + str(QUEUE_PPI_BASE_DEF) + ")", default = QUEUE_PPI_BASE_DEF, type = str)
    parser.add_argument("-r", "--rand_seed_mon_sleep", dest = "rand_seed_mon_sleep", help = "rand_seed_mon_sleep value (default is " + str(RAND_SEED_MON_SLEEP_DEF) + ")", default = RAND_SEED_MON_SLEEP_DEF, type = int)
    parser.add_argument("-i", "--index_prod", dest = "index_prod", help = "index_prod value (default is " + str(INDEX_PROD_DEF) + ")", default = INDEX_PROD_DEF, type = int)
    # parser.add_argument("-T", "--tilde_T", dest = "tilde_T", help = "tilde_T value (default is " + str(TILDE_T_DEF) + ")", default = TILDE_T_DEF, type = str)
    parser.add_argument("redishost", help = "redishost", type = str)
    parser.add_argument("redisport", help = "redisport", type = str)
    parser.add_argument("dbenvname", help = "dbenvname", type = str)
    parser.add_argument("dbenvuser", help = "dbenvuser", type = str)
    parser.add_argument("dbenvpassword", help = "dbenvpassword", type = str)
    parser.add_argument("dbenvhost", help = "dbenvhost", type = str)
    parser.add_argument("dbenvport", help = "dbenvport", type = str)

    args = parser.parse_args()

    if args.queue_ppo_base == "":
        print("Error: queue_ppo_base (-q) must not be the empty string")
        sys.exit(2)

    if args.queue_ppi_base == "":
        print("Error: queue_ppi_base (-Q) must not be the empty string")
        sys.exit(2)

    if args.index_prod < 0:
        print("Error: index_prod (-i) must not be negative")
        sys.exit(2)

    return args
    
main()
