import common_lib
import sys
import argparse
import datetime
import os
import json
import redis
import more_itertools
import random
import math

def main():
    args = parse_args()
    common_lib.print_cmd_line(sys.argv)
    common_lib.print_all_options(args)
    common_lib.init_rand_mon_sleep(args.rand_seed_mon_sleep)
    (conn_env, db_env) = common_lib.db_connect("dbname=%s host=%s user=%s password=%s port=%s" %(args.dbenvname, args.dbenvhost, args.dbenvuser, args.dbenvpassword, args.dbenvport))
    rc = redis.Redis(host=args.redishost, port=args.redisport, decode_responses=True)
    index_ship_orig = args.index_ship
    while True:
        if rc.llen(args.queue_si_base + str(args.index_ship)) > 0:
            X = rc.rpop(args.queue_si_base + str(args.index_ship))
            Xar = X.split()
            i = int(Xar[0])
            n = int(Xar[2])
            o = int(Xar[1])
            T = get_time(db_env, i, index_ship_orig)
            rc.lpush(args.queue_so_base + str(args.index_ship), "sent")
            common_lib.monitored_sleep_check(rc, db_env, "shipper_env" + str(args.index_ship), fixed_time=random.choice(T))
            rc.lpush(args.queue_soa_base + str(args.index_ship), str(o) + " arrived")
        #without this, deadlock!
        common_lib.monitored_sleep_check(rc, db_env, "shipper_env" + str(args.index_ship), cause=["main_sleep"])

def get_time(db_env, item_id, index_ship):
    sql = "SELECT ship_time FROM item_shipper_other_info WHERE item_id = %d AND shipper_id = %d" %(item_id, index_ship)
    res = common_lib.exec_and_retrieve(db_env, sql)
    return [r["ship_time"] for r in res]

def parse_args():
    QUEUE_SOA_BASE_DEF = "work:queue:SOA"
    QUEUE_SO_BASE_DEF = "work:queue:SO"
    QUEUE_SI_BASE_DEF = "work:queue:SI"
    # TILDE_T_DEF = str([0.1*i for i in range(1, 11)]).replace(", ", ":").replace("[", "").replace("]", "")
    INDEX_SHIP_DEF = 0
    RAND_SEED_MON_SLEEP_DEF = 10

    parser = argparse.ArgumentParser(description='Command arguments')
    parser.add_argument("-r", "--rand_seed_mon_sleep", dest = "rand_seed_mon_sleep", help = "rand_seed_mon_sleep value (default is " + str(RAND_SEED_MON_SLEEP_DEF) + ")", default = RAND_SEED_MON_SLEEP_DEF, type = int)
    parser.add_argument("-a", "--queue_soa_base", dest = "queue_soa_base", help = "queue_soa_base value (default is " + str(QUEUE_SOA_BASE_DEF) + ")", default = QUEUE_SOA_BASE_DEF, type = str)
    parser.add_argument("-q", "--queue_so_base", dest = "queue_so_base", help = "queue_so_base value (default is " + str(QUEUE_SO_BASE_DEF) + ")", default = QUEUE_SO_BASE_DEF, type = str)
    parser.add_argument("-Q", "--queue_si_base", dest = "queue_si_base", help = "queue_si_base value (default is " + str(QUEUE_SI_BASE_DEF) + ")", default = QUEUE_SI_BASE_DEF, type = str)
    parser.add_argument("-i", "--index_ship", dest = "index_ship", help = "index_ship value (default is " + str(INDEX_SHIP_DEF) + ")", default = INDEX_SHIP_DEF, type = int)
    parser.add_argument("redishost", help = "redishost", type = str)
    parser.add_argument("redisport", help = "redisport", type = str)
    parser.add_argument("dbenvname", help = "dbenvname", type = str)
    parser.add_argument("dbenvuser", help = "dbenvuser", type = str)
    parser.add_argument("dbenvpassword", help = "dbenvpassword", type = str)
    parser.add_argument("dbenvhost", help = "dbenvhost", type = str)
    parser.add_argument("dbenvport", help = "dbenvport", type = str)

    args = parser.parse_args()

    if args.queue_soa_base == "":
        print("Error: queue_soa_base (-a) must not be the empty string")
        sys.exit(2)

    if args.queue_so_base == "":
        print("Error: queue_so_base (-q) must not be the empty string")
        sys.exit(2)

    if args.queue_si_base == "":
        print("Error: queue_si_base (-Q) must not be the empty string")
        sys.exit(2)

    if args.index_ship < 0:
        print("Error: index_ship (-i) must not be negative")
        sys.exit(2)

    return args
    
main()
