#include <hiredis/hiredis.h>
#include <string.h>
#include <stdlib.h>
#include "aux.h"
#include <assert.h>
#include <math.h>
#include "clib.h"
#include <pqxx/pqxx> 

void manage_req_from_customers(redisContext *conn, pqxx::connection *C)
{
  static char sql[1 << 21];
  sprintf(sql, "SELECT channel_code FROM customer");
  unsigned ells_size, ells_i;
  int *ells;
  {
    pqxx::nontransaction N(*C);
    pqxx::result R(N.exec(sql));
    //monitored_sleep(conn, "customer", 0, 1, "select");
    only_advance_time_monitored_sleep(conn, "customer", 1, "select");
    ells_size = R.size();
    ells = (int *)calloc(ells_size, sizeof(int));
    ells_i = 0;
    for (pqxx::result::const_iterator c = R.begin(); c != R.end(); ++c) {
      ells[ells_i++] = c[0].as<int>();
    }
  }
  for (ells_i = 0; ells_i < ells_size; ells_i++) {
    int ell = ells[ells_i];
    redisReply *reply = (redisReply *)redisCommand(conn, "LLEN %s%d", QUEUE_CO_BASE, ell);
    // monitored_sleep(conn, "customer", 0, 1, "poll");
    only_advance_time_monitored_sleep(conn, "customer", 1, "poll");
    long long int tmp = reply->integer;
    freeReplyObject(reply);
    if (tmp) {
      char *msg[3];
      reply = (redisReply *)redisCommand(conn, "RPOP %s%d", QUEUE_CO_BASE, ell);
      // monitored_sleep(conn, "customer", 0, 1, "recv");
      only_advance_time_monitored_sleep(conn, "customer", 1, "recv");
      msg[0] = strdup(reply->str);
      freeReplyObject(reply);
      msg[1] = strtok(msg[0], " ");
      msg[2] = strtok((char *)0, " ");
      if (!msg[2]) {
	reply = (redisReply *)redisCommand(conn, "LPUSH %s %s", QUEUE_CP, msg[1]);
	// monitored_sleep(conn, "customer", 0, 1, "send");
	only_advance_time_monitored_sleep(conn, "customer", 1, "send");
	freeReplyObject(reply);
	msg[1] = blocking_read(conn, QUEUE_PC, "customer");
	reply = (redisReply *)redisCommand(conn, "LPUSH %s%d %s", QUEUE_CI_BASE, ell, msg[1]);
	// monitored_sleep(conn, "customer", 0, 1, "send");
	only_advance_time_monitored_sleep(conn, "customer", 1, "send");
	freeReplyObject(reply);
	free(msg[1]);
      }
      else {
	char *further, *curr_time_psql = get_curr_time_psql();
	further = strtok((char *)0, " ");
	sprintf(sql, "INSERT INTO order_from_customer(customer_id, item_id, how_many, processed, date_placed, date_due) VALUES(%d, %s, %s, FALSE, %s, %s + INTERVAL '%s SECONDS')", ell, msg[1], msg[2], curr_time_psql, curr_time_psql, further);
	pqxx::work W(*C);
	W.exec(sql);
	W.commit();
	// monitored_sleep(conn, "customer", 0, 1, "insert");
	only_advance_time_monitored_sleep(conn, "customer", 1, "insert");
	reply = (redisReply *)redisCommand(conn, "LPUSH %s%d OK", QUEUE_CI_BASE, ell);
	// monitored_sleep(conn, "customer", 0, 1, "send");
	only_advance_time_monitored_sleep(conn, "customer", 1, "send");
	freeReplyObject(reply);
      }
      free(msg[0]);
    }
  }
  free(ells);
}
