#include <hiredis/hiredis.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "aux.h"
#include <assert.h>
#include <math.h>
#include "clib.h"
#include <pqxx/pqxx>

static void report(redisContext *conn, pqxx::connection *C, const char *sql, const char *reason, FILE *f)
{
  static char sql_upd[1 << 10];
  int *save, i = 0;
  {
    pqxx::nontransaction N(*C);
    pqxx::result R(N.exec(sql));
    // monitored_sleep(conn, "ext_monitor", 0, 1, "select");
    only_advance_time_monitored_sleep(conn, "ext_monitor", 1, "select");
    save = (int *)calloc(R.size(), sizeof(int));
    for (pqxx::result::const_iterator c = R.begin(); c != R.end(); ++c) { 
      fprintf(f, "%d|%d|%d|%d|%s|%s|%s|%s|%s|%s\n", c[0].as<int>(), c[1].as<int>(), c[2].as<int>(), c[3].as<int>(), c[4].as<std::string>().c_str(), c[5].as<std::string>().c_str(), c[6].is_null()? "NULL" : c[6].as<std::string>().c_str(), c[7].is_null()? "NULL" : c[7].as<std::string>().c_str(), c[8].is_null()? "NULL" : c[8].as<std::string>().c_str(), reason);
      fflush(f);
      save[i++] = c[0].as<int>();
    }
  }
  for (int j = 0; j < i; j++){
    sprintf(sql_upd, "UPDATE order_from_customer SET reported = TRUE WHERE id = %d", save[j]);
    pqxx::work W(*C);
    W.exec(sql_upd);
    W.commit();
    // monitored_sleep(conn, "ext_monitor", 0, 1, "update");
    only_advance_time_monitored_sleep(conn, "ext_monitor", 1, "update");
  }
}

void aux_monitor(redisContext *conn, pqxx::connection *C, FILE *f)
{
  double elapsed = 0;
  static char sql[1 << 10];
  sprintf(sql, "SELECT id, customer_id, item_id, how_many, processed, date_placed, date_due, date_shipped, date_arrived FROM order_from_customer WHERE date_arrived > date_due AND NOT(reported)");
  report(conn, C, sql, "arr>due", f);
  sprintf(sql, "SELECT id, customer_id, item_id, how_many, processed, date_placed, date_due, date_shipped, date_arrived FROM order_from_customer WHERE %s > date_due AND NOT(reported)", get_curr_time_psql());
  report(conn, C, sql, "now>due", f);
}
