#include <hiredis/hiredis.h>
#include <string.h>
#include <stdlib.h>
#include "aux.h"
#include <assert.h>
#include <math.h>
#include "clib.h"
#include <pqxx/pqxx> 

#define DBG_HERE 0
#define DBG_HERE2 0

void manage_req_from_cp(redisContext *conn, pqxx::connection *C)
{
  redisReply *reply = (redisReply *)redisCommand(conn, "LLEN %s", QUEUE_CP);
  // monitored_sleep(conn, "producer", 0, 1, "poll");
  only_advance_time_monitored_sleep(conn, "producer", 1, "poll");
  long long int tmp = reply->integer;
  print_all(DBG_HERE2, "\t1\n");
  freeReplyObject(reply);
  if (tmp) {
    unsigned item_id;
    reply = (redisReply *)redisCommand(conn, "RPOP %s", QUEUE_CP);
    print_all(DBG_HERE2, "\t2\n");
    // monitored_sleep(conn, "producer", 0, 1, "recv");
    only_advance_time_monitored_sleep(conn, "producer", 1, "recv");
    item_id = atoi(reply->str);
    freeReplyObject(reply);
    unsigned *producer_ids;
    unsigned producer_ids_len = get_producers(conn, C, item_id, &producer_ids);
    print_all(DBG_HERE2, "\t3 %d\n", producer_ids_len);
    unsigned m = 0;
    char tmp[1 << 10];
    for (unsigned i = 0; i < 2*producer_ids_len; i += 2) {
      print_all(DBG_HERE2, "\t4\n");
      reply = (redisReply *)redisCommand(conn, "LPUSH %s%d %d", QUEUE_PII_BASE, producer_ids[i], item_id);
      print_all(DBG_HERE2, "\t5\n");
      // monitored_sleep(conn, "producer", 0, 1, "send");
      only_advance_time_monitored_sleep(conn, "producer", 1, "send");
      freeReplyObject(reply);
      print_all(DBG_HERE2, "\t6\n");
      sprintf(tmp, "%s%d", QUEUE_POI_BASE, producer_ids[i]);
      print_all(DBG_HERE2, "\t7.1 %s\n", tmp);
      char *tmp2 = blocking_read(conn, tmp, "producer");
      print_all(DBG_HERE2, "\t7.2\n");
      m += atoi(tmp2);
      free(tmp2);
    }
    print_all(DBG_HERE2, "\t8\n");
    reply = (redisReply *)redisCommand(conn, "LPUSH %s %u", QUEUE_PC, m);
    print_all(DBG_HERE2, "\t9\n");
    // monitored_sleep(conn, "producer", 0, 1, "send");
    only_advance_time_monitored_sleep(conn, "producer", 1, "send");
    freeReplyObject(reply);
    print_all(DBG_HERE2, "\t10\n");
    free(producer_ids);
  }
  print_all(DBG_HERE2, "\t100\n");
}

static unsigned get_producers(redisContext *conn, pqxx::connection *C, int item_id, unsigned **producer_ids)
{
  static char sql[1 << 10];
  if (item_id != -1)
    sprintf(sql, "SELECT channel_code, producer_id FROM item_producer INNER JOIN producer p ON producer_id = p.id WHERE item_id = %d", item_id);
  else
    sprintf(sql, "SELECT channel_code, id FROM producer");
  pqxx::nontransaction N(*C);
  pqxx::result R(N.exec(sql));
  // monitored_sleep(conn, "producer", 0, 1, "select");
  only_advance_time_monitored_sleep(conn, "producer", 1, "select");
  *producer_ids = (unsigned *)calloc(2*R.size(), sizeof(unsigned));
  unsigned i = 0;
  for (pqxx::result::const_iterator c = R.begin(); c != R.end(); ++c) { 
    (*producer_ids)[i] = c[0].as<int>();
    (*producer_ids)[i + 1] = c[1].as<int>();
    i += 2;
  }
  return i/2;
}

void manage_orders(redisContext *conn, pqxx::connection *C)
{
  static char sql[1 << 21];
  int *save_res;
  unsigned save_res_size, save_res_i;
  {
    sprintf(sql, "SELECT id, item_id, how_many FROM order_from_customer WHERE NOT(processed)");
    pqxx::nontransaction N(*C);
    pqxx::result R(N.exec(sql));
    // monitored_sleep(conn, "producer", 0, 1, "select");
    only_advance_time_monitored_sleep(conn, "producer", 1, "select");
    save_res_size = R.size();
    save_res = (int *)calloc(3*save_res_size, sizeof(int));
    save_res_i = 0;
    for (pqxx::result::const_iterator c = R.begin(); c != R.end(); ++c) {
      save_res[save_res_i++] = c[0].as<int>();
      save_res[save_res_i++] = c[1].as<int>();
      save_res[save_res_i++] = c[2].as<int>();
    }
  }
  for (save_res_i = 0; save_res_i < save_res_size; save_res_i += 3) {
    pqxx::work W(*C);
    sprintf(sql, "UPDATE order_from_customer SET processed = TRUE WHERE id = %d", save_res[save_res_i]);
    W.exec(sql);
    W.commit();
    // monitored_sleep(conn, "producer", 0, 1, "update");
    only_advance_time_monitored_sleep(conn, "producer", 1, "update");
    int item_id = save_res[save_res_i + 1];
    unsigned *producer_ids;
    unsigned producer_ids_len = get_producers(conn, C, item_id, &producer_ids);
    unsigned m = 0, i;
    for (i = 0; i < 2*producer_ids_len; i += 2) {
      sprintf(sql, "%d %d", item_id, save_res[save_res_i + 2] - m);
      redisReply *reply = (redisReply *)redisCommand(conn, "LPUSH %s%d %s", QUEUE_PII_BASE, producer_ids[i], sql);
      // monitored_sleep(conn, "producer", 0, 1, "send");
      only_advance_time_monitored_sleep(conn, "producer", 1, "send");
      freeReplyObject(reply);
      sprintf(sql, "%s%d", QUEUE_POI_BASE, producer_ids[i]);
      char *mp_str = blocking_read(conn, sql, "producer");
      unsigned mp = atoi(mp_str);
      free(mp_str);
      if (mp > 0) {
	sprintf(sql, "INSERT INTO storage(order_id, producer_id, how_many) VALUES(%d, %d, %d)", save_res[save_res_i], producer_ids[i + 1], mp);
	pqxx::work W2(*C);
	W2.exec(sql);
	W2.commit();
	// monitored_sleep(conn, "producer", 0, 1, "insert");
	only_advance_time_monitored_sleep(conn, "producer", 1, "insert");
	m += mp;
      }
      if (m >= save_res[save_res_i + 2])
	break;
    }
    if (m < save_res[save_res_i + 2]) {
      unsigned a = (save_res[save_res_i + 2] - m)/producer_ids_len;
      unsigned b = (save_res[save_res_i + 2] - m)%producer_ids_len;
      if (a != 0) {
	for (i = 0; i < 2*producer_ids_len; i += 2) {
	  sprintf(sql, "%d %d %d", item_id, save_res[save_res_i], a);
	  redisReply *reply = (redisReply *)redisCommand(conn, "LPUSH %s%d %s", QUEUE_PIP_BASE, producer_ids[i], sql);
	  // monitored_sleep(conn, "producer", 0, 1, "send");
	  only_advance_time_monitored_sleep(conn, "producer", 1, "send");
	  freeReplyObject(reply);
	}
      }
      for (i = 0; i < 2*b; i += 2) {
	sprintf(sql, "%d %d 1", item_id, save_res[save_res_i]);
	redisReply *reply = (redisReply *)redisCommand(conn, "LPUSH %s%d %s", QUEUE_PIP_BASE, producer_ids[i], sql);
	// monitored_sleep(conn, "producer", 0, 1, "send");
	only_advance_time_monitored_sleep(conn, "producer", 1, "send");
	freeReplyObject(reply);
      }
    }
    free(producer_ids);
  }
  free(save_res);
}

void manage_producers(redisContext *conn, pqxx::connection *C)
{
  int ret = 0;
  unsigned *producer_ids;
  unsigned producer_ids_len = get_producers(conn, C, -1, &producer_ids);
  for (unsigned i = 0; i < 2*producer_ids_len; i += 2) {
    redisReply *reply = (redisReply *)redisCommand(conn, "LLEN %s%d", QUEUE_POP_BASE, producer_ids[i]);
    // monitored_sleep(conn, "producer", 0, 1, "poll");
    only_advance_time_monitored_sleep(conn, "producer_prod_ans", 1, "poll");
    long long int tmp = reply->integer;
    freeReplyObject(reply);
    if (tmp) {
      char sql[1 << 10];
      unsigned order_id, n;
      reply = (redisReply *)redisCommand(conn, "RPOP %s%d", QUEUE_POP_BASE, producer_ids[i]);
      // monitored_sleep(conn, "producer", 0, 1, "recv");
      only_advance_time_monitored_sleep(conn, "producer_prod_ans", 1, "recv");
      print_all(DBG_HERE, "DBG: received %s from POP%d from index %u; tmp was %lld\n", reply->str, producer_ids[i], i, tmp);
      char *tmp3 = strdup(reply->str);
      freeReplyObject(reply);
      char *tmp2 = strtok(tmp3, " ");
      order_id = atoi(tmp2);
      tmp2 = strtok((char *)0, " ");
      n = atoi(tmp2);
      pqxx::work W(*C);
      sprintf(sql, "INSERT INTO storage(order_id, producer_id, how_many) VALUES(%d, %d, %d)", order_id, producer_ids[i + 1], n);
      W.exec(sql);
      W.commit();
      // monitored_sleep(conn, "producer", 0, 1, "insert");
      only_advance_time_monitored_sleep(conn, "producer_prod_ans", 1, "insert");
      ret = 1;
    }
  }
  free(producer_ids);
}
