-- This script must be executed by a user having superuser privileges

create database main_db_website;

create role main_db_website_user with password 'main_db_website_user_password' login;

\c main_db_website;

create table item(
	id serial,
	descr varchar(100),
	constraint item_pkey primary key(id)
);

create table producer(
	id serial,
	descr varchar(100),
	channel_code integer not null unique check(channel_code >= 0),
	constraint producer_pkey primary key(id)
);

create table customer(
	id serial,
	descr varchar(100),
	channel_code integer not null unique check(channel_code >= 0),
	constraint customer_pkey primary key(id)
);

create table shipper(
	id serial,
	channel_code integer not null unique check(channel_code >= 0),
	descr varchar(100),
	constraint shipper_pkey primary key(id)
);

-- each customer may order any item, each shipper may ship any item, only some producers may produce an item
create table item_producer(
	id serial,
	producer_id integer not null,
	item_id integer not null,
	constraint item_producer_ref1 foreign key(producer_id) references producer(id),
	constraint item_producer_ref2 foreign key(item_id) references item(id),
	constraint item_producer_pkey primary key(id)
);

create table order_from_customer(
	id serial,
	customer_id integer not null,
	item_id integer not null,
	how_many integer not null,
	processed boolean not null default false,
	reported boolean not null default false,
	date_placed timestamp without time zone not null,
	date_due timestamp without time zone not null,
	date_shipped timestamp without time zone,
	date_arrived timestamp without time zone,
	constraint order_from_customer_ref1 foreign key(customer_id) references customer(id),
	constraint order_from_customer_ref2 foreign key(item_id) references item(id),
	constraint order_from_customer_pkey primary key(id)
);

create table storage(
	id serial,
	order_id integer not null,
	producer_id integer not null,
	how_many integer not null,
	constraint storage_ref1 foreign key(producer_id) references producer(id),
	constraint storage_ref2 foreign key(order_id) references order_from_customer(id),
	constraint storage_pkey primary key(id)
);

grant select on item to main_db_website_user;
grant insert on item to main_db_website_user;
grant update on item to main_db_website_user;
grant delete on item to main_db_website_user;

grant select on item_id_seq to main_db_website_user;
grant update on item_id_seq to main_db_website_user;
grant usage on item_id_seq to main_db_website_user;

grant select on producer to main_db_website_user;
grant insert on producer to main_db_website_user;
grant update on producer to main_db_website_user;
grant delete on producer to main_db_website_user;

grant select on producer_id_seq to main_db_website_user;
grant update on producer_id_seq to main_db_website_user;
grant usage on producer_id_seq to main_db_website_user;

grant select on item_producer to main_db_website_user;
grant insert on item_producer to main_db_website_user;
grant update on item_producer to main_db_website_user;
grant delete on item_producer to main_db_website_user;

grant select on item_producer_id_seq to main_db_website_user;
grant update on item_producer_id_seq to main_db_website_user;
grant usage on item_producer_id_seq to main_db_website_user;

grant select on customer to main_db_website_user;
grant insert on customer to main_db_website_user;
grant update on customer to main_db_website_user;
grant delete on customer to main_db_website_user;

grant select on customer_id_seq to main_db_website_user;
grant update on customer_id_seq to main_db_website_user;
grant usage on customer_id_seq to main_db_website_user;

grant select on order_from_customer to main_db_website_user;
grant insert on order_from_customer to main_db_website_user;
grant update on order_from_customer to main_db_website_user;
grant delete on order_from_customer to main_db_website_user;

grant select on order_from_customer_id_seq to main_db_website_user;
grant update on order_from_customer_id_seq to main_db_website_user;
grant usage on order_from_customer_id_seq to main_db_website_user;

grant select on shipper to main_db_website_user;
grant insert on shipper to main_db_website_user;
grant update on shipper to main_db_website_user;
grant delete on shipper to main_db_website_user;

grant select on shipper_id_seq to main_db_website_user;
grant update on shipper_id_seq to main_db_website_user;
grant usage on shipper_id_seq to main_db_website_user;

grant select on storage to main_db_website_user;
grant insert on storage to main_db_website_user;
grant update on storage to main_db_website_user;
grant delete on storage to main_db_website_user;

grant select on storage_id_seq to main_db_website_user;
grant update on storage_id_seq to main_db_website_user;
grant usage on storage_id_seq to main_db_website_user;
