#include <hiredis/hiredis.h>
#include <string.h>
#include <stdlib.h>
#include "aux.h"
#include <assert.h>
#include <math.h>
#include "clib.h"
#include <pqxx/pqxx> 

#define DBG_HERE 0
#define DBG_HERE2 0

unsigned get_shippers(redisContext *conn, pqxx::connection *C, unsigned **shipper_ids)
{
  static char sql[1 << 10];
  sprintf(sql, "SELECT channel_code FROM shipper");
  pqxx::nontransaction N(*C);
  pqxx::result R(N.exec(sql));
  // monitored_sleep(conn, "shipper", 0, 1, "select");
  only_advance_time_monitored_sleep(conn, "shipper", 1, "select");
  *shipper_ids = (unsigned *)calloc(R.size(), sizeof(unsigned));
  unsigned i = 0;
  for (pqxx::result::const_iterator c = R.begin(); c != R.end(); ++c) { 
    (*shipper_ids)[i] = c[0].as<int>();
    i++;
  }
  return R.size();
}

void manage_req_from_shippers(redisContext *conn, pqxx::connection *C, unsigned *shipper_ids, char *shipper_stat, unsigned shipper_ids_len)
{
  static char sql[1 << 21];
  for (int i = 0; i < shipper_ids_len; i++) {
    if (!shipper_stat[i])
      continue;
    int ell = shipper_ids[i];
    redisReply *reply = (redisReply *)redisCommand(conn, "LLEN %s%d", QUEUE_SOA_BASE, ell);
    // monitored_sleep(conn, "shipper", 0, 1, "poll");
    only_advance_time_monitored_sleep(conn, "shipper", 1, "poll");
    long long int tmp = reply->integer;
    freeReplyObject(reply);
    if (tmp) {
      shipper_stat[i] = 0;
      char *msg[3];
      reply = (redisReply *)redisCommand(conn, "RPOP %s%d", QUEUE_SOA_BASE, ell);
      // monitored_sleep(conn, "shipper", 0, 1, "recv");
      only_advance_time_monitored_sleep(conn, "shipper", 1, "recv");
      msg[0] = strdup(reply->str);
      freeReplyObject(reply);
      msg[1] = strtok(msg[0], " ");
      msg[2] = strtok((char *)0, " ");
      if (msg[2]) {
	// print_all(DBG_HERE2, "DBG: curr_time manage_req_from_shippers is %s\n", get_curr_time_psql());
	sprintf(sql, "UPDATE order_from_customer SET date_arrived = %s WHERE id = %s", get_curr_time_psql(), msg[1]);
	{
	  pqxx::work W(*C);
	  W.exec(sql);
	  W.commit();
	  // monitored_sleep(conn, "shipper", 0, 1, "update");
	  only_advance_time_monitored_sleep(conn, "shipper", 1, "update");
	}
      }
      free(msg[0]);
    }
  }
}

void manage_orders(redisContext *conn, pqxx::connection *C, unsigned *shipper_ids, char *shipper_stat, unsigned shipper_ids_len)
{
  static char sql[1 << 21];
  int *save_res;
  unsigned save_res_size, save_res_i;
  {
    sprintf(sql, "SELECT id, item_id, how_many FROM order_from_customer WHERE processed");
    pqxx::nontransaction N(*C);
    pqxx::result R(N.exec(sql));
    // monitored_sleep(conn, "shipper", 0, 1, "select");
    only_advance_time_monitored_sleep(conn, "shipper", 1, "select");
    save_res_size = R.size();
    save_res = (int *)calloc(3*save_res_size, sizeof(int));
    save_res_i = 0;
    for (pqxx::result::const_iterator c = R.begin(); c != R.end(); ++c) {
      save_res[save_res_i++] = c[0].as<int>();
      save_res[save_res_i++] = c[1].as<int>();
      save_res[save_res_i++] = c[2].as<int>();
    }
  }
  for (save_res_i = 0; save_res_i < save_res_size; save_res_i += 3) {
    sprintf(sql, "SELECT st.how_many FROM storage st INNER JOIN order_from_customer ofd ON st.order_id = ofd.id WHERE order_id = %d AND date_shipped IS NULL", save_res[save_res_i]);
    unsigned m;
    {
      pqxx::nontransaction N(*C);
      pqxx::result R(N.exec(sql));
      //monitored_sleep(conn, "shipper", 0, 1, "select");
      only_advance_time_monitored_sleep(conn, "shipper", 1, "select");
      print_all(DBG_HERE, "DBG: %lu %d\n", R.size(), save_res[save_res_i]);
      m = 0;
      for (pqxx::result::const_iterator c = R.begin(); c != R.end(); ++c) 
	m += c[0].as<int>();
    }
    if (m >= save_res[save_res_i + 2]) {
      for (int i = 0; i < shipper_ids_len; i++) {
	if (shipper_stat[i])
	  continue;
	shipper_stat[i] = 1;
	sprintf(sql, "%d %d %d", save_res[save_res_i + 1], save_res[save_res_i], save_res[save_res_i + 2]);
	redisReply *reply = (redisReply *)redisCommand(conn, "LPUSH %s%d %s", QUEUE_SI_BASE, shipper_ids[i], sql);
	// monitored_sleep(conn, "shipper", 0, 1, "send");
	only_advance_time_monitored_sleep(conn, "shipper", 1, "send");
	freeReplyObject(reply);
	sprintf(sql, "%s%d", QUEUE_SO_BASE, shipper_ids[i]);
	blocking_read(conn, sql, "shipper");
	sprintf(sql, "UPDATE order_from_customer SET date_shipped = %s WHERE id = %d", get_curr_time_psql(), save_res[save_res_i]);
	{
	  pqxx::work W(*C);
	  W.exec(sql);
	  W.commit();
	  // monitored_sleep(conn, "shipper", 0, 1, "update");
	  only_advance_time_monitored_sleep(conn, "shipper", 1, "update");
	}
	break;
      }
    }
  }
}
